# -*- Makefile -*-
# $Id: latex.mk.in.in,v 1.43 2004/03/18 20:22:46 dan Exp $ 
#
# Makefile fragment for handling LaTeX projects
#
# Copyright (c) 2001, 2002, 2003 Dan McMahill
# All rights reserved.
#
# This code is derived from software written by Dan McMahill
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#        This product includes software developed by Dan McMahill
#  4. The name of the author may not be used to endorse or promote products
#     derived from this software without specific prior written permission.
# 
#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
#  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
#  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#  SUCH DAMAGE.
#


#######################################
#
# README:
#
# To use this makefile fragment, set the variable:
#	NAME	= Top level project name.  Your top level LaTeX 
#		  file should be called $(NAME).tex
#
#	TEXSRCS	= Additional LaTeX input files (.tex)
#
# and add the line:
#	.include "latex.mk"
# in your top level Makefile
#
# The following variables may be overridden in your top level Makefile
#
#  BIBTEX	= the BibTeX executable
#  BIBTEX_ENV	= environment variables set when running the BibTeX executable
#  BIBEX_FLAGS	= flags passed to the BibTeX executable
#  CONVERT      = the 'convert' program which is part of ImageMagick
#  DVIPDFM      = the dvipdfm executable
#  DVIPDFM_ENV  = environment variables set when running the dvipdfm executable
#  DVIPDFM_FLAGS= flags passed to the dvipdfm executable
#  DVIPS	= the dvips executable
#  DVIPS_FLAGS	= flags passed to the dvips executable
#  GV		= a postscript previewer executable
#  GV_FLAGS	= flags passed to the postscript previewer executable
#  JPG2EPS      = program and flags used to convert JPEG files to EPS
#  LATEX	= the LaTeX executable
#  LATEX_ENV	= environment variables set when running the LaTeX executable
#  LATEX_FLAGS	= flags passed to the LaTeX executable
#  LATEX2HTML   = LaTeX2HTML executable
#  LATEX2HTML_ENV
#               = environment variables set when running LATEX2HTML
#  LATEX2HTML_FLAGS
#               = flags passed to LATEX2HTML
#  LATEX2RTF    = LaTeX2RTF executable
#  LATEX2RTF_ENV
#               = environment variables set when running LATEX2RTF
#  LATEX2RTF_FLAGS
#               = flags passed to LATEX2RTF
#  LPR		= executable to print postscript files
#  LPR_FLAGS	= flags passed to the executable to print postscript files
#  PDFLATEX     = pdflatex executable
#  PDFLATEX_ENV = environment variables set when running the PDFLaTeX executable
#  PDFLATEX_FLAGS= flags passed to the PDFLaTeX executable
#  PNG2EPS      = program and flags used to convert PNG files to EPS
#  PS2PDF	= ps2pdf executable
#  PS2PDF_FLAGS	= flags passed to the ps2pdf executable
#  VIEWPDF      = PDF viewer
#  VIEWPDF_FLAGS= PDF viewer flags
#  XDVI		= a .dvi previewer executable
#  XDVI_FLAGS	= flags passed to the .dvi previewer executable
#
#  USE_DVIPDFM  = set to use dvipdfm to create pdf files from dvi
#                 instead of using dvips and ps2pdf
#  USE_PDFLATEX = set to use pdflatex to create pdf files from .tex
#                 files instead of using latex, dvips and ps2pdf
#
#######################################


MAKECONF?=	/usr/local/etc/latex-mk.conf
USER_MAKECONF?=	${HOME}/.latex-mk.conf

.if exists(${MAKECONF})
.include "${MAKECONF}"
.endif

.if exists(${USER_MAKECONF})
.include "${USER_MAKECONF}"
.endif


# Make sure we set at least 1 way for creating html files
.if !defined(USE_HEVEA)
USE_LATEX2HTML=	YES
.endif

# The location of the latex-mk .mk and .gmk files
# we do this instead of hard coding so the package
# can be tested before installation by setting this
# variable in a test config file
LATEX_MK_DIR?=	/usr/local/share/latex-mk

AWK?=		/usr/bin/awk
BIBTEX?=	/usr/local/bin/bibtex
CONVERT?=	convert
DVIPDFM?=	/usr/local/bin/dvipdfm
DVIPDFM_ENV?=	
DVIPDFM_FLAGS+=	
DVIPS?=		/usr/local/bin/dvips
DVIPS_FLAGS+=	-j0
ECHO?=		/bin/echo
ENV_PROG?=	/usr/bin/env
FALSE?=		/usr/bin/false
FIND?=		/usr/bin/find
GREP?=		/usr/bin/grep
GV?=		gv
GV_FLAGS?=
HACHA?=		hacha
HACHA_ENV?=
HACHA_FLAGS?=
HEVEA?=		hevea
HEVEA_ENV?=
HEVEA_FLAGS?=	-fix
IMAGEN?=	imagen
JPG2EPS?=	${CONVERT} 
LATEX_MK?=	/usr/local/bin/latex-mk
LATEX_MK_FLAGS?=
LATEX?=		/usr/local/bin/latex
LATEX_ENV?=	
LATEX2HTML?=	latex2html
LATEX2HTML_ENV?=
LATEX2HTML_FLAGS?=-image_type png -local_icons -show_section_numbers 
LATEX2RTF?=	latex2rtf
LATEX2RTF_ENV?=
LATEX2RTF_FLAGS?=
LPR?=		/usr/bin/lpr
LPR_FLAGS?=	
PDFLATEX?=	/usr/local/bin/pdflatex
PDFLATEX_ENV?=	
PDFLATEX_FLAGS?=	
PNG2EPS?=	${CONVERT} 
POST_BIBTEX_HOOK?=
RM?=		/bin/rm
RMDIR?=		/bin/rmdir
PS2PDF?=	ps2pdf
PS2PDF_FLAGS?=	
XDVI?=		/usr/local/bin/xdvi
XDVI_FLAGS?=	
VIEWPDF?=	${GV}
VIEWPDF_FLAGS?=	${GV_FLAGS}


DRAFT_STAMP_PS?=	draft_stamp.ps
DATED_DRAFT_PS?=	${LATEX_MK_DIR}/dated_draft.ps

.if !target(default)
.PHONY: default
default: all
.endif

.if !target(all)
.MAIN: all
.PHONY: all
all:: dvi
all:: view-all
.endif

REAL_LATEX_ENV+=	LATEX=${LATEX}
REAL_LATEX_ENV+=	LATEX_FLAGS=${LATEX_FLAGS}
REAL_LATEX_ENV+=	BIBTEX=${BIBTEX}
REAL_LATEX_ENV+=	BIBTEX_FLAGS=${BIBTEX_FLAGS}
REAL_LATEX_ENV+=	POST_BIBTEX_HOOK=${POST_BIBTEX_HOOK}
REAL_LATEX_ENV+=	${LATEX_ENV}

REAL_PDFLATEX_ENV+=	PDFLATEX=${PDFLATEX}
REAL_PDFLATEX_ENV+=	PDFLATEX_FLAGS=${PDFLATEX_FLAGS}
REAL_PDFLATEX_ENV+=	BIBTEX=${BIBTEX}
REAL_PDFLATEX_ENV+=	BIBTEX_FLAGS=${BIBTEX_FLAGS}
REAL_PDFLATEX_ENV+=	POST_BIBTEX_HOOK=${POST_BIBTEX_HOOK}
REAL_PDFLATEX_ENV+=	${PDFLATEX_ENV}

# Include other .mk files often times used with LaTeX

#######################################
#
# Check for tgif
#
#######################################

# Check for TGIFSRCS and TGIFDIRS

.if defined(TGIFSRCS) || defined(TGIFDIRS)
_USE_TGIF_MK=	YES
.endif


# Check for proj_TGIFSRCS and proj_TGIFDIRS

.for __tmp__ in ${NAME}
.if defined(${__tmp__}_TGIFSRCS) || defined(${__tmp__}_TGIFDIRS)
_USE_TGIF_MK=	YES
.endif
.endfor


# pull in tgif.[g]mk if needed

.if defined(_USE_TGIF_MK)
.include "${LATEX_MK_DIR}/tgif.mk"
.endif

#######################################
#
# Check for xfig
#
#######################################

# Check for XFIGSRCS and XFIGDIRS

.if defined(XFIGSRCS) || defined(XFIGDIRS)
_USE_XFIG_MK=	YES
.endif


# Check for proj_XFIGSRCS and proj_XFIGDIRS

.for __tmp__ in ${NAME}
.if defined(${__tmp__}_XFIGSRCS) || defined(${__tmp__}_XFIGDIRS)
_USE_XFIG_MK=	YES
.endif
.endfor


# pull in xfig.[g]mk if needed

.if defined(_USE_XFIG_MK)
.include "${LATEX_MK_DIR}/xfig.mk"
.endif

#######################################
#
# Targets
#
#######################################

#######################################
# target:  view_<proj>
#######################################

.for __tmp__ in ${NAME}
.PHONY: view_${__tmp__}
view_${__tmp__}: ${__tmp__}.dvi
	$(XDVI) $(XDVI_FLAGS) ${__tmp__}.dvi
DEFAULTVIEW?=	view_${__tmp__}
ALLVIEW+=	view_${__tmp__}
.endfor


#######################################
# target:  viewps_<proj>
#######################################

.for __tmp__ in ${NAME}
.PHONY: viewps_${__tmp__}
viewps_${__tmp__}: ${__tmp__}.ps
	$(GV) $(GV_FLAGS)  ${__tmp__}.ps
DEFAULTVIEWPS?=	viewps_${__tmp__}
ALLVIEWPS+=	viewps_${__tmp__}
.PHONY: viewps_${__tmp__}-draft
viewps_${__tmp__}-draft: ${__tmp__}-draft.ps
	$(GV) $(GV_FLAGS)  ${__tmp__}-draft.ps
DEFAULTVIEWPS_DRAFT?=	viewps_${__tmp__}-draft
ALLVIEWPS_DRAFT+=	viewps_${__tmp__}-draft
${__tmp__}-draft.ps: ${__tmp__}.dvi ${DATED_DRAFT_PS}
	sed "s;DATE;`date`;g" ${DATED_DRAFT_PS} > ${DRAFT_STAMP_PS}
	$(DVIPS) -h ${DRAFT_STAMP_PS} $(DVIPS_FLAGS) -o $@ ${__tmp__}.dvi
.endfor


#######################################
# target:  viewpdf_<proj>
#######################################

.for __tmp__ in ${NAME}
.PHONY: viewpdf_${__tmp__}
viewpdf_${__tmp__}: ${__tmp__}.pdf
	$(VIEWPDF) $(VIEWPDF_FLAGS)  ${__tmp__}.pdf
DEFAULTVIEWPDF?=	viewpdf_${__tmp__}
ALLVIEWPDF+=	viewpdf_${__tmp__}
.PHONY: viewpdf_${__tmp__}-draft
viewpdf_${__tmp__}-draft: ${__tmp__}-draft.pdf
	$(VIEWPDF) $(VIEWPDF_FLAGS)  ${__tmp__}-draft.pdf
DEFAULTVIEWPDF_DRAFT?=	viewpdf_${__tmp__}-draft
ALLVIEWPDF_DRAFT+=	viewpdf_${__tmp__}-draft
.endfor


#######################################
# target:  print_<proj>
#######################################

.for __tmp__ in ${NAME}
.PHONY: print_${__tmp__}
print_${__tmp__}: ${__tmp__}.ps
	$(LPR) $(LPR_FLAGS) ${__tmp__}.ps
DEFAULTPRINT?=	print_${__tmp__}
ALLPRINT+=	print_${__tmp__}
.PHONY: print_${__tmp__}-draft
print_${__tmp__}-draft: ${__tmp__}-draft.ps
	$(LPR) $(LPR_FLAGS) ${__tmp__}-draft.ps
DEFAULTPRINT_DRAFT?=	print_${__tmp__}-draft
ALLPRINT_DRAFT+=	print_${__tmp__}-draft
.endfor



#######################################

ALLTEXSRCS=	${TEXSRCS}
ALLBIBTEXSRCS=	${BIBTEXSRCS}

#######################################
#
# add the dependencies for each documents .dvi file
#
#######################################

.for __tmp__ in ${NAME}
${__tmp__}.dvi:	${__tmp__}.tex ${TEXSRCS} ${${__tmp__}_TEXSRCS} ${BIBTEXSRCS} ${${__tmp__}_BIBTEXSRCS} ${OTHER} ${${__tmp__}_OTHER}
.if defined(USE_PDFLATEX)
${__tmp__}.pdf:	${__tmp__}.tex ${TEXSRCS} ${${__tmp__}_TEXSRCS} ${BIBTEXSRCS} ${${__tmp__}_BIBTEXSRCS} ${OTHER} ${${__tmp__}_OTHER}
.endif

ALLTEXSRCS+=	${__tmp__}.tex ${${__tmp__}_TEXSRCS}
ALLBIBTEXSRCS+=	${${__tmp__}_BIBTEXSRCS}
dvi:	${__tmp__}.dvi
html:	${__tmp__}.html-stamp
.PHONY: ${__tmp__}.html
${__tmp__}.html:	${__tmp__}.html-stamp
${__tmp__}.html-stamp:	${__tmp__}.dvi
ps:	${__tmp__}.ps
pdf:	${__tmp__}.pdf
ps-draft:	${__tmp__}-draft.ps
pdf-draft:	${__tmp__}-draft.pdf
${__tmp__}.rtf:	${__tmp__}.dvi
rtf:	${__tmp__}.rtf
clean::
	if [ -f ${__tmp__}.www_files ]; then  \
		for f in `${AWK} '{print $$2}' ${__tmp__}.www_files` ; do \
			if [ -f $$f ]; then ${RM} -f $$f ; fi ;\
		done ;\
	fi
	${RM} -f ${__tmp__}.www_files
.if defined(USE_LATEX2HTML)
	${RM} -f ${__tmp__}.html_dir/images.aux
	${RM} -f ${__tmp__}.html_dir/images.bbl
	${RM} -f ${__tmp__}.html_dir/images.log
	${RM} -f ${__tmp__}.html_dir/images.out
	${RM} -f ${__tmp__}.html_dir/images.tex
.endif
	if [ -d ${__tmp__}.html_dir ]; then ${RMDIR} ${__tmp__}.html_dir ; fi
.endfor



#######################################
# target:  view, view-all
#######################################

.PHONY: view
view: ${DEFAULTVIEW}
.PHONY: view-all
view-all: ${ALLVIEW}


#######################################
# target:  viewps, viewps-all
#######################################

.PHONY: viewps
viewps: ${DEFAULTVIEWPS}
.PHONY: viewps-all
viewps-all: ${ALLVIEWPS}

#######################################
# target:  viewpdf, viewpdf-all
#######################################

.PHONY: viewpdf
viewpdf: ${DEFAULTVIEWPDF}
.PHONY: viewpdf-all
viewpdf-all: ${ALLVIEWPDF}

#######################################
# target:  viewps-draft, viewps-all-draft
#######################################

.PHONY: viewps-draft
viewps-draft: ${DEFAULTVIEWPS_DRAFT}
.PHONY: viewps-all-draft
viewps-all-draft: ${ALLVIEWPS_DRAFT}

#######################################
# target:  viewpdf-draft, viewpdf-all-draft
#######################################

.PHONY: viewpdf-draft
viewpdf-draft: ${DEFAULTVIEWPDF_DRAFT}
.PHONY: viewpdf-all-draft
viewpdf-all-draft: ${ALLVIEWPDF_DRAFT}

#######################################
# target:  print, print-all
#######################################

.PHONY: print
print: ${DEFAULTPRINT}
.PHONY: print-all
print-all: ${ALLPRINT}

#######################################
# target:  print-draft, print-all-draft
#######################################

.PHONY: print-draft
print-draft: ${DEFAULTPRINT_DRAFT}
.PHONY: print-all-draft
print-all-draft: ${ALLPRINT_DRAFT}

#######################################
#
#######################################

.SUFFIXES : .tex .dvi .ps .pdf .rtf .html-stamp .png .jpg .eps

.if defined(IGNORE_LATEX_ERRORS)
LATEX_MK_FLAGS+=	--ignore-errors
.endif

#######################################
#
# Graphic file conversions
#
#######################################
.jpg.eps : 
	$(JPG2EPS) $< $@

.png.eps : 
	$(PNG2EPS) $< $@

#######################################
#
# DVI Output (latex)
#
#######################################
.PHONY: dvi
.tex.dvi : 
	$(ENV_PROG) $(REAL_LATEX_ENV) $(LATEX_MK) $(LATEX_MK_FLAGS) $<

#######################################
#
# HTML Output (latex2html or HeVeA)
#
#######################################

# note:  we have to use a time stamp file here to keep track of what files
# get created by the html conversion.  Neither latex2html nor hevea provide
# a convenient way to get a list of files to clean.  In addition, hevea
# does not currently provide a DESTDIR type functionality which is needed
# for dealing with multiple top level documents.


.PHONY: html
.tex.html-stamp : 
	touch $@1
.if defined(USE_LATEX2HTML)
	@${ECHO} LaTeX-Mk:  Creating HTML output using latex2html
	$(ENV_PROG) $(LATEX2HTML_ENV) $(LATEX2HTML) $(LATEX2HTML_FLAGS) -mkdir -dir $*.html_dir $<
.elif defined(USE_HEVEA)
	@${ECHO} LaTeX-Mk:  Creating HTML output using HeVeA
	mkdir -p $*.html_dir
	-$(ENV_PROG) $(HEVEA_ENV) $(HEVEA) $(HEVEA_FLAGS) $< 
	-$(ENV_PROG) $(HEVEA_ENV) ${IMAGEN}  `basename $< .tex`
	-$(ENV_PROG) $(HEVEA_ENV) $(HEVEA) $(HEVEA_FLAGS) $< 
.endif
	if [ ! -f $*.www_files ]; then touch $*.www_files ; fi
	for f in `${FIND} . -type f \( -name \*.html -o -name \*.htm -o -name \*.gif \
		-o -name \*.png -o -name \*.css -o -name \*.pl -o -name WARNINGS \) \
		-newer $@1 -print` ; do \
		fn=`basename $$f` ; \
		if ${GREP} $$f $*.www_files >/dev/null ; then \
			echo "$$f already recorded" ; \
		else \
			echo "recording file $$f ($*.html_dir/$$fn)" ; \
			echo "$$f $*.html_dir/$$fn" >> $*.www_files ; \
		fi ;\
	done
.if defined(USE_HEVEA)
	for f in `${AWK} '{print $$1}' $*.www_files` ; do \
		${ECHO} "Moving $$f to $*.html_dir" ; mv $$f $*.html_dir ; \
	done
.endif
	mv $@1 $@

#######################################
#
# Postscript Output (dvips)
#
#######################################
.PHONY:  ps

# if we're doing a *-draft target, then add the 
# DRAFT watermark and time stamp
.if make(*-draft)
__DRAFT=	yes
.endif

.dvi.ps :
.if defined(__DRAFT)
	sed "s;DATE;`date`;g" ${DATED_DRAFT_PS} > ${DRAFT_STAMP_PS}
	$(DVIPS) -h ${DRAFT_STAMP_PS} $(DVIPS_FLAGS) -o $*-draft.ps $<
.else
	$(DVIPS) $(DVIPS_FLAGS) -o $@ $<
.endif

#######################################
#
# PDF Output (ps2pdf)
#
#######################################
.PHONY:  pdf

.if defined(USE_DVIPDFM)
.dvi.pdf :
	$(ENV_PROG) $(DVIPDFM_ENV) $(DVIPDFM) $(DVIPDFM_FLAGS) $<
.elif defined(USE_PDFLATEX)
.tex.pdf : 
	$(ENV_PROG) $(REAL_PDFLATEX_ENV) $(LATEX_MK) --pdflatex $(LATEX_MK_FLAGS) $<
.else
.ps.pdf :
	$(PS2PDF) $(PS2PDF_FLAGS) $<
.endif

#######################################
#
# RTF (Rich Text Format) output (latex2rtf)
#
#######################################
.PHONY:  rtf

.dvi.rtf :
	$(ENV_PROG) $(LATEX2RTF_ENV) $(LATEX2RTF) $(LATEX2RTF_FLAGS) -o $@ $*.tex

#######################################
#
# Clean up (clean)
#
#######################################
CLEAN_FILES+=	${ALLTEXSRCS:.tex=.aux}
CLEAN_FILES+=	${BIBTEXSRCS:.bib=.aux}
CLEAN_FILES+=	${NAME:=.ps}
CLEAN_FILES+=	${NAME:=-draft.ps}
CLEAN_FILES+=	${NAME:=.pdf}
CLEAN_FILES+=	${NAME:=-draft.pdf}
CLEAN_FILES+=	${NAME:=.rtf}
CLEAN_FILES+=	${NAME:=.aux}
CLEAN_FILES+=	${NAME:=.log}
CLEAN_FILES+=	${NAME:=.dvi}
CLEAN_FILES+=	${NAME:=.log}
CLEAN_FILES+=	${NAME:=.bbl}
CLEAN_FILES+=	${NAME:=.blg}
CLEAN_FILES+=	${NAME:=.lof}
CLEAN_FILES+=	${NAME:=.lot}
CLEAN_FILES+=	${NAME:=.toc}
CLEAN_FILES+=	${NAME:=.out}
CLEAN_FILES+=	$(DRAFT_STAMP_PS)

# HTML generation
CLEAN_FILES+=	${NAME:=.html-stamp}
CLEAN_FILES+=	${NAME:=.www_files}

.ifdef USE_HEVEA
CLEAN_FILES+=	${NAME:=.haux}
CLEAN_FILES+=	${NAME:=.image.tex}
.endif

.ifdef USE_LATEX2HTML
CLEAN_FILES+=	${NAME:=.haux}
CLEAN_FILES+=	${NAME:=.image.tex}
.endif


.PHONY: clean

clean::

clean::
.for __tmp__ in ${NAME}
	${LATEX_MK} --clean ${__tmp__}
.endfor
	$(RM) -f *~ texput.log $(CLEAN_FILES)

.PHONY: distclean
distclean: clean

#######################################
#
# Some helpful debugging targets
#
#######################################
.PHONY: show-var
show-var:
	@${ECHO} ${${VARNAME}}

